#!/usr/bin/perl -w
################################################################################
# Windows to Linux Migration Toolkit
#-----------------------------------
# w2lmt.pm
# This perl module contains general purpose functions for the w2lmt scripts
#
# 2004-08-14  Christian B. Lahti <clahti@neomagic.com>
################################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

use strict;

################################################################################
sub lconnect {
################################################################################
# Creates connection to local LDAP server and binds as admin DN
################################################################################
	my ($server,$bindport) = @_;
	$lconn = Net::LDAP->new(
				$server,
				port => $bindport,
				version => 3,
				timeout => 60
				)
				or die "\nError contacting target LDAP server $server:$bindport";
	$lmesg = $lconn->bind ( $config{'ladn'}, password => "$lpass" );
	$lmesg->code && die "\nError binding to target LDAP server $server:$bindport\n", $lmesg->error, $config{'ladn'}, $lpass;
}

################################################################################
sub xconnect {
################################################################################
# Creates connection to windows LDAP server and binds as admin DN
################################################################################
	my $conntype = shift;
	my ($xcmd, $xbase);

	#get the naming contexts to find out our windows base DN
	if ($conntype eq "AD") {
		$xcmd = $config{'ldapsearchcmd'} . " -x -b '' -s base '(objectclass=*)' namingContexts -h $config{SourceHost} | grep 'namingContexts: DC'";
	} elsif	($conntype eq "EXCH55") {
		$xcmd = $config{'ldapsearchcmd'} . " -x -b '' -s base '(objectclass=*)' namingContexts -h $config{SourceHost} | grep 'namingContexts: ou=' | grep -v '_ABViews_'";
	} else {
		die ("Invalid SourceType specified in your config file, please check your settings!");
	}
	$xbase = `$xcmd`;
	$xbase =~ s/namingContexts: //;
	chomp $xbase;

	if (! $xbase ) {
		die "Unable to obtain naming context for $config{SourceHost}\n";
	} else {
		print "found naming context for $config{SourceHost}: $xbase\n";
	}
	#set the admin DN in config
	$config{'xbase'} = $xbase;
	if ($conntype eq "AD") {
		$config{'xadn'} = "CN=$config{SourceAdminAccount},CN=Users,$xbase";
	} else {
		$config{'xadn'} = $xbase . ", cn=" . $config{'SourceAdminAccount'};
	}
	
	#connect/bind to windows LDAP server
	$xconn = Net::LDAP->new(
				$config{'SourceHost'},
				port => '389',
				version => 3,
				timeout => 60
				)
				or die "\nError contacting target LDAP server $config{'SourceHost'}:389\n";
	$xmesg = $xconn->bind ( $config{'xadn'}, password => "$xpass" );
	$xmesg->code && die "\nError binding to target LDAP server $config{'SourceHost'}:389\n", $xmesg->error;
}


################################################################################
sub add_user_to_gid {
################################################################################
# Adds uid to passed group id, for either posix or distribution Group
################################################################################
	my ($gid,$uid) = @_;
	my ($srch, $mesg, $entry);

	#first find the group and it's type
	$srch = "(&(|(objectClass=posixGroup)(objectClass=groupOfNames))(cn=$gid))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if (!$entry) {
		warn ($gid . " group or distribution list not found!\n");
		return 1;
	} else {
		my $ref = $entry->get_value( 'objectClass', asref => 1);
		foreach my $val (@$ref) {
			if ($val eq "posixGroup") {
				$mesg = $lconn->modify($entry, add => { memberUid => $uid } );
				if ($mesg->code) {
					warn ("could not add $uid to $gid, the error was: " . $mesg->error);
					return 1;
				} else {
					return 0;
				}
			}
			if ($val eq "groupOfNames") {
				#ok, so we need the user dn
				my $uiddn = user_exists($uid);
				if (!$uiddn) {
					$uiddn = contact_exists($uid);
				}
				$mesg = $lconn->modify($entry, add => { member => $uiddn } );
				if ($mesg->code) {
					warn ("could not add $uid to $gid, the error was: " . $mesg->error);
					return 1;
				} else {
					return 0;
				}
			}
		}
	}
}

################################################################################
sub user_in_group {
################################################################################
# Determines whether a uid is a member of either posix or distribution Group
################################################################################
	my ($gid,$uid) = @_;
	my ($srch, $mesg, $entry, $ref);

	#first find the group
	$srch = "(&(|(objectClass=posixGroup)(objectClass=groupOfNames))(cn=$gid))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if ($entry) {
		my $oref = $entry->get_value( 'objectClass', asref => 1);
		foreach my $val (@$oref) {
			if ($val eq "posixGroup") {
				my $mref = $entry->get_value( 'memberUid', asref => 1);
				foreach my $val (@$mref) {
					if ($val eq $uid) {
						return 1;
					}
				}
			}
			if ($val eq "groupOfNames") {
				#ok, so we need the user dn
				my $uiddn = user_exists($uid);
				if (!$uiddn) {
					$uiddn = contact_exists($uid);
				}
				$ref = $entry->get_value( 'member', asref => 1);
				foreach my $val (@$ref) {
					if ($val eq $uiddn) {
						return 1;
					}
				}
			}
		}
	}
	return 0;
}

################################################################################
sub user_exists {
################################################################################
# Determines whether a user DN already exists
################################################################################
	my ($uid) = shift;
	my ($srch, $mesg, $entry);
	$srch = "(&(objectClass=inetOrgPerson)(uid=$uid)(objectClass=posixAccount))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if ($entry) {
		return $entry->dn();
	}
	return 0;
}

################################################################################
sub contact_exists {
################################################################################
# Determines whether a contact DN already exists
################################################################################
	my ($cn) = shift;
	my ($srch, $mesg, $entry);
	$srch = "(&(objectClass=inetOrgPerson)(cn=$cn)(!(objectClass=posixAccount)))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if ($entry) {
		return $entry->dn();
	}
	return 0;
}

################################################################################
sub group_exists {
################################################################################
# Determines whether a group DN already exists
################################################################################
	my ($gid) = shift;
	my ($srch, $mesg, $entry);
	$srch = "(&(cn=$gid)(objectClass=posixGroup))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if ($entry) {
		return $entry->dn();
	}
	return 0;
}

################################################################################
sub dgroup_exists {
################################################################################
# Determines whether a group DN already exists
################################################################################
	my ($gid) = shift;
	my ($srch, $mesg, $entry);
	$srch = "(&(cn=$gid)(objectClass=groupOfNames))";
	$mesg = $lconn->search(base=>$config{'lbase'}, filter=>$srch);
	$entry = $mesg->entry(0);
	if ($entry) {
		return $entry->dn();
	}
	return 0;
}

################################################################################
sub create_dgroup {
################################################################################
# Creates a distribution group object
################################################################################
	my ($cn, $description) = @_;
	my $dn = "cn=$cn,$config{DistributionGroupsOU},$config{lbase}";
	my $attr = [
		'objectclass'=>["groupOfNames","top"],
		'cn'=>$cn,
		'member'=>$dn,
		'description'=>$description,
		];

	my $mesg = $lconn->add( $dn, attr=>$attr );
	if ($mesg->code) {
		warn ("create_dgroup_member $dn: " . $mesg->error);
	}
	return $dn;
}

################################################################################
sub create_contact {
################################################################################
# Creates a contact object
################################################################################
	my ($cn, $givenName, $sn) = @_;
	my $dn = "cn=$cn,$config{ContactsOU},$config{lbase}";
	my $attr = [
		'objectclass'=>["inetOrgPerson","top"],
		'cn'=>$cn,
		'givenName'=>$givenName,
		'sn'=>$sn,
		];

	my $mesg = $lconn->add( $dn, attr=>$attr );
	if ($mesg->code) {
		warn ("create_dgroup_member $dn: " . $mesg->error);
	}
	return $dn;
}

1;
